var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import { startCheckAndRun } from './PrivateFunctions';
import log from './log';

import { assignFunction, isFunction, isArray, assignNumber } from './util';

var ConcurrentTasks = function ConcurrentTasks() {
    var _this = this;

    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, ConcurrentTasks);

    this.__working = false;
    this.tasks = {
        list: [],
        total: 0,
        completed: 0,
        running: 0
    };
    this.duration = {
        start: 0,
        end: 0,
        total: 0
    };

    this.start = function () {
        if (_this.__working) {
            console.warn(log('already_running'));
            return false;
        }

        if (_this.config.autoStart) {
            console.warn(log('auto_start_true'));
            return false;
        }

        startCheckAndRun.call(_this);
        return true;
    };

    this.add = function (task) {
        if (isFunction(task)) {
            var autoStart = _this.config.autoStart;

            _this.tasks.list.push(task);
            _this.tasks.total++;
            if (autoStart) {
                startCheckAndRun.call(_this);
            }
            return true;
        }

        throw new TypeError(log('add_requires_function'));
    };

    this.addMultiple = function (tasks) {
        if (isArray(tasks) && tasks.every(function (t) {
            return isFunction(t);
        })) {
            var autoStart = _this.config.autoStart;

            _this.tasks = {
                list: [].concat(_this.tasks.list, tasks),
                total: tasks.length
            };
            if (autoStart) {
                startCheckAndRun.call(_this);
            }
            return true;
        }

        throw new TypeError(log('add_multiple_requires_array_of_functions'));
    };

    this.remove = function (index) {
        _this.tasks.list.splice(index, 1);
        _this.tasks.total = _this.tasks.list.length;
    };

    this.removeAll = function () {
        _this.tasks.list = [];
        _this.tasks.total = 0;
    };

    var concurrency = config.concurrency,
        onStart = config.onStart,
        onDone = config.onDone,
        onEnd = config.onEnd,
        otherConfig = _objectWithoutProperties(config, ['concurrency', 'onStart', 'onDone', 'onEnd']);

    this.config = _extends({
        autoStart: true
    }, otherConfig);
    this.concurrency = assignNumber(concurrency, 3);
    this.onStart = assignFunction(onStart);
    this.onDone = assignFunction(onDone);
    this.onEnd = assignFunction(onEnd);
};

export { ConcurrentTasks as default };